/**
 * @file
 */

(function ($, Drupal, drupalSettings) {

  "use strict";

  Drupal.behaviors.conditional_message = {
    attach: function (context, settings) {
      // Run only once (not on ajax calls).
      if (context != document) {
        return;
      }
      // Shorthand for the configurations.
      let config = drupalSettings.conditional_message;
      let htmls = [];
      // TODO rewrite this in pure JS to avoid jQuery dependency.
      for (let key in config) {
        // Bail if user already closed the message before.
        let cmuc = 'conditionalMessageUserClosed' + key;
        if (localStorage.getItem(cmuc)) {
          continue;
        }
        (function ($, config, key, htmls) {
          queryEndpoint($, config, key, htmls);
        }).call(this, $, config, key, htmls);
      }
    }
  };
})(jQuery, Drupal, drupalSettings);


function queryEndpoint($, config, key, htmls) {
  $.ajax({
    url: config[key].base_path + "conditional_message_data_output", success: function (result) {
      // Checks to see if the message should be displayed.  Config = backend logic, Result = frontend logic.
      let check = [];
      // Check and set sessions with localStorage.
      check['session'] = true;
      let cmrs = 'conditionalMessageReadStatus' + key;
      let readStatus = localStorage.getItem(cmrs);
      if (config[key].conditions.includes('session')) {
        if (readStatus) {
          check['session'] = false;
        } else {
          localStorage.setItem(cmrs, true);
        }
      }
      // Check close button with localStorage.
      check['close'] = true;
      if (config[key].conditions.includes('close') && result[key].close) {
        let cmuc = 'conditionalMessageUserClosed' + key;
        let closeStatus = localStorage.getItem(cmuc);
        check['close'] = !closeStatus;
      }
      // Paths conditions.
      check['path'] = true;
      if (config[key].conditions.includes('path') && result[key].paths.indexOf(window.location.pathname.substr(config[key].base_path.length - 1)) < 0) {
        check['path'] = false;
      }
      // Content type conditions.
      check['type'] = true;
      if (config[key].conditions.includes('content_type') && result[key].hasOwnProperty('types')) {
        check['type'] = false;
        result[key].types.forEach(function (type) {
          let typeClass = 'page-node-type-' + type;
          if ($('body').hasClass(typeClass)) {
            check['type'] = true;
          }
        });
      }
      // Show message if all checks pass.
      if (config[key].status && result[key].display && check['session'] && check['path'] && check['type'] && check['close']) {
        // Build the message HTML.
        let html = '<div class="conditional-message" data-cm-key="'
          + key +'" style="background-color:#'
          + config[key].bg_color + '; color:#'
          + config[key].color + ';"><span>&times;</span>'
          + config[key].message + '</div>';
        htmls[key] = html;
        // Place the message in the page top or bottom.
        switch (config[key].position) {
          case  'bottom':
            $(config[key].target).append($(htmls[key]).addClass('conditional-message-bottom'));
            break;

          default:
            $(config[key].target).prepend($(htmls[key]).addClass('conditional-message-top'));
        }
        // Close button.
        $('.conditional-message span').on('click', function () {
          let cmKey = $(this).parent().data('cm-key');
          $(this).parent().remove();
          let cmuc = 'conditionalMessageUserClosed' + cmKey;
          localStorage.setItem(cmuc, true);
        });
      }
    }
  });
}
